@@ -68,7 +68,6 @@ static int
 parse_geojson_coord(json_object *poObj, int *hasz, POINTARRAY *pa)
 {
 	POINT4D pt;
-	int iType = 0;
 
 	LWDEBUGF(3, "parse_geojson_coord called for object %s.", json_object_to_json_string( poObj ) );
 
@@ -176,12 +175,11 @@ parse_geojson_linestring(json_object *geojson, int *hasz,  int root_srid)
 static LWGEOM*
 parse_geojson_polygon(json_object *geojson, int *hasz,  int root_srid)
 {
-	LWGEOM *geom;
-	POINTARRAY **ppa;
+	POINTARRAY **ppa = NULL;
 	json_object* rings = NULL;
+	json_object* points = NULL;
 	int i = 0, j = 0;
-	int nRings = 0;
-	int nPoints = 0;
+	int nRings = 0, nPoints = 0;
 
 	rings = findMemberByName( geojson, "coordinates" );
 	if ( ! rings ) 
@@ -192,48 +190,48 @@ parse_geojson_polygon(json_object *geojson, int *hasz,  int root_srid)
 
 	if ( json_type_array != json_object_get_type(rings) )
 	{
-		geojson_lwerror("The 'coordinates' in GeoJSON string are not an array", 4);
+		geojson_lwerror("The 'coordinates' in GeoJSON are not an array", 4);
 		return NULL;
 	}
-	
+
 	nRings = json_object_array_length( rings );
 
+	/* No rings => POLYGON EMPTY */
 	if ( ! nRings )
 	{
 		return (LWGEOM *)lwpoly_construct_empty(root_srid, 0, 0);
 	}
-
-	ppa = (POINTARRAY**) lwalloc(sizeof(POINTARRAY*));
-
-	json_object* points = NULL;
-	ppa[0] = ptarray_construct_empty(1, 0, 1);
-	points = json_object_array_get_idx( rings, 0 );
-	nPoints = json_object_array_length( points );
-
-	for (i=0; i < nPoints; i++ )
+	
+	for ( i = 0; i < nRings; i++ )
 	{
-		json_object* coords = NULL;
-		coords = json_object_array_get_idx( points, i );
-		parse_geojson_coord(coords, hasz, ppa[0]);
-	}
-
-	for(i = 1; i < nRings; ++i)
-	{
-		int nPoints;
-		ppa = (POINTARRAY**) lwrealloc((POINTARRAY *) ppa, sizeof(POINTARRAY*) * (i + 1));
+		points = json_object_array_get_idx(rings, i);
+		if ( ! points || json_object_get_type(points) != json_type_array )
+		{
+			geojson_lwerror("The 'coordinates' in GeoJSON ring are not an array", 4);
+			return NULL;
+		}
+		nPoints = json_object_array_length(points);
+		
+		/* Skip empty rings */
+		if ( nPoints == 0 ) continue;
+		
+		if ( ! ppa )
+			ppa = (POINTARRAY**)lwalloc(sizeof(POINTARRAY*) * nRings);
+		
 		ppa[i] = ptarray_construct_empty(1, 0, 1);
-		points = json_object_array_get_idx( rings, i );
-		nPoints = json_object_array_length( points );
-		for (j=0; j < nPoints; j++ )
+		for ( j = 0; j < nPoints; j++ )
 		{
 			json_object* coords = NULL;
 			coords = json_object_array_get_idx( points, j );
 			parse_geojson_coord(coords, hasz, ppa[i]);
 		}
-	}
-
-	geom = (LWGEOM *) lwpoly_construct(root_srid, NULL, nRings, ppa);
-	return geom;
+	}	
+	
+	/* All the rings were empty! */
+	if ( ! ppa )
+		return (LWGEOM *)lwpoly_construct_empty(root_srid, 0, 0);
+	
+	return (LWGEOM *) lwpoly_construct(root_srid, NULL, nRings, ppa);
 }
 
 static LWGEOM*
